package rwth.i2.ltlrv.formula.base;

import rwth.i2.ltlrv.formula.interfaze.IFormula;

/**
 * CommutativeBinaryFormula - Abstract superclass of all binary formulae which
 * are commutative. Overrides {@link java.lang.Object#equals(java.lang.Object)}
 * and {@link java.lang.Object#hashCode()} accordingly.
 * 
 * @author Eric Bodden
 */
public abstract class CommutativeBinaryFormula extends BinaryFormula {

    /**
     * @param subformula1
     * @param subformula2
     */
    public CommutativeBinaryFormula(IFormula subformula1, IFormula subformula2) {
        super(subformula1, subformula2);
    }

    /**
     * Implements an equality check based on the assumption that this formula constructor
     * is commutative.
     * @see Object#equals(java.lang.Object)
     */
    @Override
    public boolean equals(Object oth) {
        if(super.equals(oth)) {
            return true;
        }
        
        if(!(oth instanceof CommutativeBinaryFormula)) {
            return false;
        }
        
        CommutativeBinaryFormula other = (CommutativeBinaryFormula)oth;        
        return (other.getSubformula1().equals(subformula1)
            && other.getSubformula2().equals(subformula2)) ||
               (other.getSubformula1().equals(subformula2)
            && other.getSubformula2().equals(subformula1));
    }

    /**
     * Calculates a hash code based on the assumption that this formula constructor
     * is commutative.
     * @see Object#hashCode()
     */
    @Override
    public int hashCode() {
        return subformula1.hashCode() + subformula2.hashCode();
    }
    
}
