package rwth.i2.ltlrv.formula.impl;

import rwth.i2.ltlrv.afastate.interfaze.IAFAState;
import rwth.i2.ltlrv.formula.base.CommutativeBinaryFormula;
import rwth.i2.ltlrv.formula.interfaze.IEquivalent;
import rwth.i2.ltlrv.formula.interfaze.IFormula;

/**
 * Equivalent - Implements the <i>equivalent</i> relation as subformula.
 * The relation is commutative and thus extends
 * {@link rwth.i2.ltlrv.formula.base.CommutativeBinaryFormula}.
 * 
 * @author Eric Bodden
 */
public class Equivalent extends CommutativeBinaryFormula implements IEquivalent {

    /**
     * Constructs an <i>equivalent</i> relation for the two subformulae.
     * @param subFormula1 left subformula
     * @param subFormula2 right subformula
     */
    public Equivalent(IFormula subFormula1, IFormula subFormula2) {
        super(subFormula1,subFormula2);
    }

    /**
     * {@inheritDoc}
     */
    public IAFAState negationNormalForm() {
        //s1 <-> s2 = s1 -> s2 && s2 -> s1
        return factory.And(
                factory.Impl(
                        subformula1,
                        subformula2
                        ),
                factory.Impl(
                        subformula2,
                        subformula1
                        )
        ).negationNormalForm();                        
    }

    /**
     * {@inheritDoc}
     */
    public String symbol() {
        return "<->";
    }
}
