package rwth.i2.ltlrv.formula.interfaze;

import rwth.i2.ltlrv.afastate.interfaze.IAFAState;
import rwth.i2.ltlrv.afastate.interfaze.IAnd;
import rwth.i2.ltlrv.afastate.interfaze.IOr;
import rwth.i2.ltlrv.afastate.interfaze.IProposition;
import rwth.i2.ltlrv.afastate.interfaze.IRelease;
import rwth.i2.ltlrv.afastate.interfaze.IUntil;

/**
 * @author Eric Bodden
 *
 * Reflects a formula in the system. Some formulae can also
 * be a state in an alternating finite automaton (AFA).
 * @see IAFAState
 */
public abstract interface IFormula {

    /**
     * Produces a formula in negation normal form, where negations
     * occur only in front of propositions. This form also holds
     * only IUntil, IRelease, IAnd, IOr, INot, IProposition subformulae.
     * This must return a copy of the original formula. 
     * The major work is done in
     * {@link rwth.i2.ltlrv.afastate.impl.Not#negationNormalForm()}.       
     * @return the formula in negation normal form
     * @see IUntil
     * @see IRelease
     * @see IAnd
     * @see IOr
     * @see IProposition
     */
    public IAFAState negationNormalForm();

    /**
     * Returns the symbol for this term constructor.
     * Propositions should return their label.
     * All symbols, including those for nullary term constructors
     * should be distinct. 
     * @return The symbol for this term constructor. 
     */
    public String symbol();
}
