package rwth.i2.ltlrv.management;

import rwth.i2.ltlrv.afastate.impl.And;
import rwth.i2.ltlrv.afastate.impl.FF;
import rwth.i2.ltlrv.afastate.impl.Next;
import rwth.i2.ltlrv.afastate.impl.Not;
import rwth.i2.ltlrv.afastate.impl.Or;
import rwth.i2.ltlrv.afastate.impl.Proposition;
import rwth.i2.ltlrv.afastate.impl.Release;
import rwth.i2.ltlrv.afastate.impl.TT;
import rwth.i2.ltlrv.afastate.impl.Until;
import rwth.i2.ltlrv.afastate.interfaze.IAnd;
import rwth.i2.ltlrv.afastate.interfaze.IFF;
import rwth.i2.ltlrv.afastate.interfaze.IIfClosure;
import rwth.i2.ltlrv.afastate.interfaze.INext;
import rwth.i2.ltlrv.afastate.interfaze.INot;
import rwth.i2.ltlrv.afastate.interfaze.IOr;
import rwth.i2.ltlrv.afastate.interfaze.IProposition;
import rwth.i2.ltlrv.afastate.interfaze.IRelease;
import rwth.i2.ltlrv.afastate.interfaze.ITT;
import rwth.i2.ltlrv.afastate.interfaze.IUntil;
import rwth.i2.ltlrv.data.WeakValuesMap;
import rwth.i2.ltlrv.formula.impl.Equivalent;
import rwth.i2.ltlrv.formula.impl.Finally;
import rwth.i2.ltlrv.formula.impl.Globally;
import rwth.i2.ltlrv.formula.impl.Implies;
import rwth.i2.ltlrv.formula.interfaze.IEquivalent;
import rwth.i2.ltlrv.formula.interfaze.IFinally;
import rwth.i2.ltlrv.formula.interfaze.IGlobally;
import rwth.i2.ltlrv.formula.interfaze.IImplies;
import rwth.i2.ltlrv.formula.interfaze.IFormula;

/**
 * FormulaFactory - Default implemetation of
 * {@link rwth.i2.ltlrv.management.IFormulaFactory}.
 * Returns instances from the default implementing classes.
 * 
 * @author Eric Bodden
 */
public class FormulaFactory implements IFormulaFactory {

    /** The unique instance of ITT */
    private final static ITT tt = new TT();

    /** The unique instance of IFF */
    private final static IFF ff = new FF();
    
    /**
     * {@inheritDoc}
     */
    public IAnd And(IFormula formula1, IFormula formula2) {
        return new And(formula1, formula2);
    }

    /**
     * {@inheritDoc}
     */
    public IEquivalent Eq(IFormula formula1, IFormula formula2) {
        return new Equivalent(formula1, formula2);
    }

    /**
     * {@inheritDoc}
     */
    public IFinally F(IFormula formula) {
        return new Finally(formula);
    }

    /**
     * {@inheritDoc}
     */
    public IGlobally G(IFormula formula) {
        return new Globally(formula);
    }

    /**
     * {@inheritDoc}
     */
    public IImplies Impl(IFormula formula1, IFormula formula2) {
        return new Implies(formula1, formula2);
    }

    /**
     * {@inheritDoc}
     */
    public INext X(IFormula formula) {
        return new Next(formula);
    }

    /**
     * {@inheritDoc}
     */
    public INot Not(IFormula formula) {
        return new Not(formula);
    }

    /**
     * {@inheritDoc}
     */
    public IOr Or(IFormula formula1, IFormula formula2) {
        return new Or(formula1,formula2);
    }

    /**
     * {@inheritDoc}
     */
    public IProposition Proposition(String label, String[] formals, IIfClosure[] ifClosures) {
        return new Proposition(label, formals, ifClosures);
    }

    /**
     * {@inheritDoc}
     */
    public IProposition Proposition(String label, String... formals) {
        return new Proposition(label, formals, new IIfClosure[] {});
    }

    /**
     * {@inheritDoc}
     */
    @Deprecated
    public IProposition Proposition(String label, WeakValuesMap<String, Object> bindings) {
        return new Proposition(label, bindings);
    }    

    /**
     * {@inheritDoc}
     */
    public IRelease Release(IFormula formula1, IFormula formula2) {
        return new Release(formula1,formula2);
    }

    /**
     * {@inheritDoc}
     */
    public IUntil Until(IFormula formula1, IFormula formula2) {
        return new Until(formula1, formula2);
    }

    /**
     * {@inheritDoc}
     */
    public ITT TT() {
        return tt;
    }

    /**
     * {@inheritDoc}
     */
    public IFF FF() {
        return ff;
    }
}
