package rwth.i2.ltlrv.management.listeners;

import java.awt.Toolkit;

import rwth.i2.ltlrv.management.Configuration;
import rwth.i2.ltlrv.management.VerificationRuntime.Listener;

public class ConsoleListener implements Listener {

    /**
     * {@inheritDoc}
     */
    public void notifyRegistered(String formulaId, Thread associatedThread, Configuration initialConfig) {
        System.err.println("---------------------------------------------------------------------------------------------");
        System.err.println("REGISTERED FORMULA '"+formulaId+"' (Thread ("+System.identityHashCode(associatedThread)+")) "+initialConfig);
        System.err.println("---------------------------------------------------------------------------------------------");
        System.err.println("\n\n");
    }

    /**
     * {@inheritDoc}
     */
    public void notifyUpdate(String formulaId, Thread associatedThread, Configuration newConfig) {
//        System.err.println("---------------------------------------------------------------------------------------------");
//        System.err.println("UPDATED FORMULA '"+formulaId+"' (Thread ("+System.identityHashCode(associatedThread)+")) "+newConfig);
        if(newConfig.isFF()) {
            System.err.println("FORMULA '"+formulaId+"' FALSIFIED!");
            Toolkit.getDefaultToolkit().beep();
        } else if(newConfig.isTT()) {
            System.err.println("FORMULA '"+formulaId+"' SATISFIED!");
        }
//        System.err.println("---------------------------------------------------------------------------------------------");
//        System.err.println("\n\n");
    }

	/**
	 * {@inheritDoc}
	 */
	public void notifyTearDown(String formulaId, Configuration config) {		
        System.err.println("---------------------------------------------------------------------------------------------");
		if(!config.isFinal()) {
            System.err.println("FORMULA '"+formulaId+"' IN NONFINAL STATE AT SHUTDOWN! ");
            System.err.println("  --> "+config);
		} else {
            System.err.println("FORMULA '"+formulaId+"' IN FINAL STATE AT SHUTDOWN. ");
        }
        System.err.println("---------------------------------------------------------------------------------------------");
        System.err.println("\n\n");
	}

    /**
     * {@inheritDoc}
     */
    public void notifyOnUserCauseException(String formulaId, Thread associatedThread, String ifExpression, Throwable exception, Configuration config) {
        System.err.println("---------------------------------------------------------------------------------------------");
        System.err.println("FORMULA '"+formulaId+"' (Thread ("+System.identityHashCode(associatedThread)+"))");
        System.err.println("CAUSED AN EXCEPTION:");
        System.err.println("  --> "+exception.getMessage());
        System.err.println("WHILE EVALUATING:");
        System.err.println("  --> "+ifExpression);
        System.err.println("LAST GOOD CONFIGURATION");
        System.err.println("  --> "+config);
        System.err.println("FORMULA WILL BE REMOVED FROM FURTHER EVALUATION");
        System.err.println("---------------------------------------------------------------------------------------------");
        System.err.println("\n\n");
    }

}
