package rwth.i2.ltlrv.management;

import rwth.i2.ltlrv.afastate.interfaze.IIfClosure;
import rwth.i2.ltlrv.afastate.interfaze.IProposition;
import rwth.i2.ltlrv.data.WeakValuesHashMap;
import rwth.i2.ltlrv.data.WeakValuesMap;


/**
 * EvalIfTestEquality
 * 
 * @author Eric Bodden
 */
@SuppressWarnings("all")
public class EvalIfTestUnequality extends EvalAbstractEvaluationTest {

    private final static IIfClosure[] closureAUnEq = new IIfClosure[] {
        new IIfClosure() {

            public boolean satisfiedUnderBindings(WeakValuesMap<String, Object> currentBinding) throws UserCausedException {
                Object a = currentBinding.get("a");
                Object b = currentBinding.get("b");
                return a!=b;
            }
            
            public String toString() { return "a!=b"; }

			public String[] variableNames() {
				return new String[] {"a","b"};
			}
            
        }            
    };

    private final static IProposition PROP_A = f.Proposition("p", new String[] {"a"});
    private final static IProposition PROP_B_BueqA = f.Proposition("p", new String[] {"b"}, closureAUnEq );
    
    public EvalIfTestUnequality() {
        super( f.G(f.Impl(p_a,f.X(f.F(f.Proposition("p", new String[] {"a","b"}, closureAUnEq ))))) ); 
    }
    
    public void testEqualityPositiveSameObject() {
        assertTrue(config.isFinal());

        WeakValuesMap<String, Object> bindings = new WeakValuesHashMap<String,Object>();
        bindings.put("a","o1");
        props.add(PROP_A.specializeBindings(bindings));
        config.transition(props);
        assertFalse(config.isFinal());
        
        props.clear();
        
        bindings = new WeakValuesHashMap<String,Object>();
        bindings.put("b","o1");
        props.add(PROP_B_BueqA.specializeBindings(bindings));
        config.transition(props);
        assertFalse(config.isFinal());
    }
    
    public void testEqualityNegativeWrongObject() {
        assertTrue(config.isFinal());

        WeakValuesMap<String, Object> bindings = new WeakValuesHashMap<String,Object>();
        bindings.put("a","o1");
        props.add(PROP_A.specializeBindings(bindings));
        config.transition(props);
        assertFalse(config.isFinal());
        
        props.clear();
        
        bindings = new WeakValuesHashMap<String,Object>();
        bindings.put("b","o2");
        props.add(PROP_B_BueqA.specializeBindings(bindings));
        config.transition(props);
        assertTrue(config.isFinal());
    }

//    private WeakValuesMap<String, Object> addUnBound(WeakValuesMap<String, Object> input, String id) {
//        WeakValuesMap<String, Object> result = new WeakValuesHashMap<String,Object>();
//        result.putAll(input);
//        result.put(id,IProposition.UNBOUND);
//        return result;
//    }    
     
}
