package rwth.i2.ltlrv.management;

import rwth.i2.ltlrv.afastate.interfaze.IIfClosure;
import rwth.i2.ltlrv.afastate.interfaze.IProposition;
import rwth.i2.ltlrv.data.PropositionSet;
import rwth.i2.ltlrv.data.WeakValuesHashMap;
import rwth.i2.ltlrv.data.WeakValuesMap;


/**
 * EvalLOR2ndHalfTest - This tests represents athe second half of the Lock Order Reversal test.
 * We generate input bindings which represent the expected bindings after the first half and then
 * apply the formula of the second half.
 * 
 * @author Eric Bodden
 */
@SuppressWarnings("all")
public class EvalLOR2ndHalfTest extends EvalAbstractEvaluationTest {

    private final static IIfClosure[] ifIUnEqJ = new IIfClosure[] {
            new IIfClosure() {

                public boolean satisfiedUnderBindings(WeakValuesMap<String, Object> currentBinding) throws UserCausedException {
                    Object i = currentBinding.get("i");
                    Object j = currentBinding.get("j");
                    return i!=j;
                }
                
                public String toString() { return "i!=j"; }

				public String[] variableNames() {
					return new String[] {"j"};
				}
                
            }            
    };
    
    private final static IProposition PROP_LOCK_JX;
    private final static IProposition PROP_LOCK_JY_JuI;
    private final static IProposition PROP_UNLOCK_JY;
    private final static IProposition PROP_LOCK_JX_JuI;
    static {
    	final WeakValuesMap<String, Object> bindings = new WeakValuesHashMap<String, Object>();
    	bindings.put("i","1");
    	bindings.put("j","2");
    	bindings.put("x","1");
    	bindings.put("y","2");
		
    	PROP_LOCK_JX =
    		f.Proposition("lock", new String[] {"j","x"}).specializeBindings(bindings);    	
    	PROP_LOCK_JY_JuI =
    		f.Proposition("lock_if", new String[] {"j","y","i"}, ifIUnEqJ).specializeBindings(bindings);
    	PROP_UNLOCK_JY =
    		f.Proposition("unlock", new String[] {"j","y"}).specializeBindings(bindings);
    	PROP_LOCK_JX_JuI =
    		f.Proposition("lock_if", new String[] {"j","x","i"}, ifIUnEqJ).specializeBindings(bindings);		
    }
    	
    
    
    public EvalLOR2ndHalfTest() {
        super( f.G(f.Not(f.Until(f.Not(PROP_LOCK_JX),f.And(PROP_LOCK_JY_JuI,f.Until(f.Not(PROP_UNLOCK_JY),PROP_LOCK_JX_JuI))))) ); 
    }
    
    public void test() {
    	assertTrue(config.isFinal());
    	
    	WeakValuesMap<String, Object> bindings;
    	PropositionSet props = new PropositionSet();

    	bindings = new WeakValuesHashMap<String, Object>();
    	bindings.put("j","2");
    	bindings.put("y","2");
    	props.add(
    			f.Proposition("lock_if", new String[] {"j","y","i"}, ifIUnEqJ).specializeBindings(bindings)
    	);
    	config.transition(props);
    	assertTrue(config.isFinal());
    	
    	
    	bindings = new WeakValuesHashMap<String, Object>();
    	bindings.put("j","2");
    	bindings.put("x","1");
    	props.add(
    			f.Proposition("lock_if", new String[] {"j","x","i"}, ifIUnEqJ).specializeBindings(bindings)
    	);    	
    	config.transition(props);
    	assertTrue(config.isFF());
    }
    
         
}
